indexing
	description: "A tree node which displays the structure of an arbitrary object"
	author: "Peter Brandt and Martin Geisler"
	date: "$Date: 2005-06-26 21:26:20 +0200 (Sun, 26 Jun 2005) $"
	revision: "$Revision: 107 $"

class
	SPYGLASS_TREE_ITEM
	
inherit
	EV_TREE_ITEM
	
	INTERNAL
		undefine
			default_create,
			copy,
			is_equal
		end

create
	make,
	make_internal

feature {NONE} -- Initialization

	make (a_name: STRING; an_object: ANY) is
			-- Creation procedure
			--| Creates child nodes corresponding to the fields in `an_object'
		do
			make_internal (a_name, an_object)
			create_children
		end		

	make_internal (a_name: STRING; an_object: ANY) is
			-- Creation procedure
			--| Does not create child nodes corresponding to the fields in `an_object'.
			--| One must call `create_children' manually to create child nodes for the initial SPYGLASS_TREE_ITEM
		require
			name_not_void: a_name /= Void
		do
			default_create
			name := a_name
			object := an_object
			basic_type := True
			
			if object = Void then
				type := "NONE"
				value := " = Void"
			else
				-- It is important to not simply use
				-- `object.generating_type' since object might be
				-- violating its class invariant, and thus the call to
				-- `generating_type' would fail.  These two functions 
				-- give the same result but does so without calling 
				-- any feature on `object'.
				type := class_name_of_type (dynamic_type (object))
				if is_instance_of (object, Numeric) or is_instance_of (object, Boolean_ref) then
					value := " = " + object.out
				elseif is_instance_of (object, Character_ref) or is_instance_of (object, Wide_character_ref) then
					value := " = '" + object.out + "'"
				elseif is_instance_of (object, String) then
					-- Use only a substring here?
					value := " = %"" + object.out + "%""
				else
					value := ""
					basic_type := False
					-- Use the "kamikaze" version because we want the children to
					-- be added at most once
					expand_actions.extend_kamikaze (agent create_grand_children)
				end
			end
			set_text (name + ": " + type + value)
		end

feature -- Measurement

	object: ANY
			-- Object being examined

	name: STRING
			-- Chosen name for `object'

	type: STRING
			-- Generating type of `object'

	value: STRING
			-- Value of `object'

feature {SPYGLASS_TREE_ITEM} -- Implementation

	create_children is
			-- If `object' is non-basic, create child nodes for all references
		local
			i: INTEGER
			child: SPYGLASS_TREE_ITEM
			tuple: TUPLE
			chain: CHAIN [ANY]
		do
			if not basic_type then
				-- Test for TUPLE and add fields for each entry
				tuple ?= object
				if tuple /= Void then
					from
						i := tuple.lower
					until
						i > tuple.upper
					loop
						create child.make_internal ("tuple_item_" + i.out, tuple.item (i))
						extend (child)
						i := i + 1
					end
				end

				-- Test for CHAIN and add fields for each item
				chain ?= object
				if chain /= Void then
					from
						i := 1
					until
						i > chain.count
					loop
						create child.make_internal("chain_item_" + i.out, chain.i_th (i))
						extend (child)
						i := i + 1
					end
				end

				-- Now add any real fields
				from
					i := 1
				until
					i > field_count (object)
				loop
					create child.make_internal (field_name (i, object), field (i, object))
					extend (child)
					i := i + 1
				end
			end
		end

feature {NONE} -- Implementation

	create_grand_children is
			-- Create children for our children
		local
			child: SPYGLASS_TREE_ITEM
		do
			from
				start
			until
				after
			loop
				-- Assignment attempt because item comes from `EV_DYNAMIC_LIST'
				child ?= item
				if child /= Void then
					child.create_children
				end
				forth
			end
		end

	basic_type: BOOLEAN
			-- Is `object' of a basic type?

	Numeric: INTEGER is
			-- Dynamic type ID of `NUMERIC'
		once
			Result := dynamic_type_from_string ("NUMERIC")
		end
	
	Character_ref: INTEGER is
			-- Dynamic type ID of `CHARACTER_REF'
		once
			Result := dynamic_type_from_string ("CHARACTER_REF")
		end
	
	Wide_character_ref: INTEGER is
			-- Dynamic type ID of `WIDE_CHARACTER_REF'
		once
			Result := dynamic_type_from_string ("WIDE_CHARACTER_REF")
		end
	
	String: INTEGER is
			-- Dynamic type ID of `STRING'
		once
			Result := dynamic_type_from_string ("STRING")
		end
	
	Boolean_ref: INTEGER is
			-- Dynamic type ID of `BOOLEAN_REF'
		once
			Result := dynamic_type_from_string ("BOOLEAN_REF")
		end

invariant
	name_not_void: name /= Void	

end -- class SPYGLASS_TREE_ITEM
