indexing
	description: "Window that allows browsing of a list of Saved States"
	author: "Peter Brandt and Martin Geisler"
	date: "$Date: 2005-06-26 22:57:19 +0200 (Sun, 26 Jun 2005) $"
	revision: "$Revision: 108 $"

class
	SAVED_STATE_INSPECTOR

inherit
	EV_TITLED_WINDOW
		redefine
			initialize
		end

	INTERNAL
		undefine
			default_create,
			copy
		end

	INTERFACE_SHARED
		undefine
			default_create,
			copy
		end

create
	make

feature {NONE} -- Initialization

	make (states: LIST [SAVED_STATE]) is
			-- Creation procedure
		require
			states_not_empty: states /= Void and then not states.is_empty
		do
			saved_states := states
			default_create
		end
		
	initialize is
			-- Create widgets
		do
			Precursor {EV_TITLED_WINDOW}

			create main_container
			main_container.set_border_width (Widget_margin)
			main_container.set_padding (Widget_margin)
			put (main_container)

			build_state_frame
			build_buttons		

			-- Set the current saved state to the last one, then update the widgets
			saved_states.finish
			sync_widgets

			previous_button.select_actions.extend (agent previous_button_action)
			next_button.select_actions.extend (agent next_button_action)

			close_request_actions.extend (agent destroy)
			
			set_height (Window_height)
			set_width (Window_width)
			set_title (Saved_state_inspector_window_title)
			show				
		end
		
	build_state_frame is
			-- Populate the Saved State frame with widgets
		local
			frame_box: EV_VERTICAL_BOX
			widget: EV_WIDGET
		do
			create state_frame.make_with_text (Saved_state_frame_text)
			main_container.extend (state_frame)
			
			create frame_box
			frame_box.set_border_width (Widget_margin)
			frame_box.set_padding_width (Widget_margin)
			state_frame.put (frame_box)
			
			create routine_label
			create class_label
			create spyglass_tree

			widget := left_align (class_label)
			frame_box.extend (widget)
			frame_box.disable_item_expand (widget)
			
			widget := left_align (routine_label)
			frame_box.extend (widget)
			frame_box.disable_item_expand (widget)

			frame_box.extend (spyglass_tree)
		ensure
			routine_label_not_void: routine_label /= Void
			class_label_not_void: class_label /= Void
			spyglass_tree_not_void: spyglass_tree /= Void
		end
	
	build_buttons is
			-- Create buttons
		local
			hbox: EV_HORIZONTAL_BOX
		do
			create hbox
			hbox.set_padding_width (Widget_margin)
			main_container.extend (hbox)
			main_container.disable_item_expand (hbox)

			create previous_button.make_with_text (Previous_saved_state_text)
			create next_button.make_with_text (Next_saved_state_text)
			
			hbox.extend (previous_button)
			hbox.extend (next_button)
		end

feature {NONE} -- Implementation

	sync_widgets is
			-- Update widgets to match `saved_states.item'
		require
			state_frame_not_void: state_frame /= Void
			routine_label_not_void: routine_label /= Void
			class_label_not_void: class_label /= Void
			spyglass_tree_not_void: spyglass_tree /= Void
		local
			i: INTEGER
			node: SPYGLASS_TREE_ITEM
			arguments_node: EV_TREE_ITEM
		do	
			-- Loop thru, adding arguments as SPYGLASS_TREE_ITEMS
			create arguments_node.make_with_text (Routine_arguments_text)
			from
				i := saved_states.item.routine_arguments.Lower
			until
				i > saved_states.item.routine_arguments.upper
			loop
				create node.make ("argument" + i.out, saved_states.item.routine_arguments.item (i))
				arguments_node.extend (node)
				i := i + 1
			end
			
			-- Make Current Object part of the tree for the saved state
			create node.make (Current_object_text, saved_states.item.current_object)
			spyglass_tree.wipe_out
			spyglass_tree.extend (arguments_node)
			spyglass_tree.extend (node)

			state_frame.set_text (Saved_state_frame_text + saved_states.index.out + "/" + saved_states.count.out)
			routine_label.set_text (Routine_label_text + saved_states.item.routine_name)
			class_label.set_text (Class_label_text + node.type)
			
			if saved_states.isfirst then
				previous_button.disable_sensitive
			else
				previous_button.enable_sensitive
			end

			if saved_states.islast then
				next_button.disable_sensitive
			else
				next_button.enable_sensitive
			end			
		end

	previous_button_action is
			-- Move to previous saved state, update widgets
		require
			previous_exists: not saved_states.isfirst
		do
			saved_states.back
			sync_widgets
		end

	next_button_action is
			-- Move to next saved state, update widgets
		require
			next_exists: not saved_states.islast
		do
			saved_states.forth
			sync_widgets
		end

	saved_states: LIST [SAVED_STATE]
		-- The states under inspection
		
feature {NONE} -- Implementation / Widgets

	main_container: EV_VERTICAL_BOX

	state_frame: EV_FRAME
			-- Frame with state specific information

	routine_label: EV_LABEL
			-- Routine name label

	class_label: EV_LABEL
			-- Class name label

	spyglass_tree: EV_TREE
			-- Tree for `SPYGLASS_TREE_ITEM' objects

	previous_button: EV_BUTTON
			-- Button for switching to previous saved state
			
	next_button: EV_BUTTON
			-- Button for switching to next saved state

feature {NONE} -- Implementation / Constants

	Window_width: INTEGER is 400
			-- Initial window width
	Window_height: INTEGER is 300
			-- Initial window height

invariant
	states_not_empty: saved_states /= Void and then not saved_states.is_empty

end -- class SAVED_STATE_INSPECTOR
